/*
 * File : 
 * 	include/aioli.h
 * 	
 * 	aIOLi is an efficient I/O library for parallel access to remote storages in SMP clusters. 
 * 	Thanks to the kernel features, our framework provides parallel I/O algorithms without 
 * 	inter-processes synchronisation mechanisms as well as a simple interface based on 
 * 	the classic UNIX system calls (creat/open/read/write/close).
 *
 * 	This file is included in the first version of our prototype aIOLi.
 * 	To know which version is currently stable please visit http://aioli.imag.fr
 * 	
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author : 
 * 	Adrien Lebre, adrien.lebre@imag.fr
 *
 * Licence : 
 * 	GPLv2	
 * 	This program is free software; you can redistribute it and/or
 * 	modify it under the terms of the GNU General Public License
 * 	as published by the Free Software Foundation; either version 2
 * 	of the License, or (at your option) any later version.
 * 	
 * 	
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program; if not, write to the Free Software
 * 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Contributors :
 * 	BULL Open Source Division - INRIA France
 * 	
 * Created :
 * 	April 2004, the 1st
 *
 * Last Modification(s) : 
 * 	April 2005
 * 	
 * Compilation :
 * 	 see Makefile
 * 	
 * Comments : 
 * 	Implement the overloaded functions called in aioli.h
 * 	in order to exchange I/O requests between user process and aIOLi daemon.
 * 	
 * 	Actually, the read_aioli make only one copy : Indeed, a shared memory segment (with mmap) is create during
 * 	the initialisation phase. aIOLi uses this shared segment to put each related data from aiolid to client part.
 * 	The post_readrqst() have just to copy from shared memory segment to the real buffer provided by read_aioli call.
 * 	Note this approach is simple but not thread safe. Indeed, if two thread make a read, the shared memory segment is n't protected.
 *
 * Todo : 
 * 	grep TODO in file ;) ..... :D 
 * 	overloaded write() call.
 * 	overloaded dup() methods (corresponds to a simple copy of the relatd index in the iobx_tab ;) )
 * 	Please secure each shared memory segment. Now the mask "0666" is used. Find a way to limit the
 * 	access to the shm only by aioli daemon.
 * 	Find a cleany way to create/setup/close/delete IPC message queue, shared memory segment.
 * 	For example if the user forgets to close all files before it exits you have to clean
 * 	each memory things ;)
 *
 * 	Warnings, for asynchrone operations like iread or iwrite find a way to check replies.
 * 	Check if a solution exist without thread concept :p. I'm not sure but I don't think create a "user"
 * 	thread without the agreement of the user process will be a great idea :D.
 * 
 * Changelog :
 * 	look into the CVS log !
 *
*/

#ifndef __AIOLI_IO_H
#define __AIOLI_IO_H 1

//#define __LIBC_OVERLOAD_ 1
#ifdef __LIBC_OVERLOAD_
// Ok, a stupid tip to overload libc. 
// Maybe, 've to make something cleaner like __libc__open but seems to be a evil way. So as a little kindly student, I eared my professor ;) and used this famous tip !!!
#define open open_aioli
#define creat creat_aioli
#define lseek lseek_aioli
#define read read_aioli
#define write write_aioli
#define close close_aioli
#endif

extern int creat_aioli (const char *pathname, mode_t mode);
extern int open_aioli (const char *pathname, int flags, ...);
extern int close_aioli (int fd);
extern off_t lseek_aioli(int fd, off_t offset, int whence);
extern size_t read_aioli (int fd, void *buf, size_t count);
extern size_t write_aioli (int fd, void *buf, size_t count);


#endif
